# -*- coding: utf-8 -*-

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.conf.urls import patterns, url

from jmb.core.admin.options import ExtendibleModelAdmin

from acomea.mailup.models import Import
from acomea.mailup.views import WizardFileSelectionView, WizardGroupsSelectionView, WizardFieldsConfigurationView


class ImportAdmin(ExtendibleModelAdmin):
    list_display = (
        'date_create', 'creator', 'mailup_list', 'status', 'file_download',
        'list_groups', 'mailup_status', 'change_db_status', 'report',
    )
    date_hierarchy = 'date_create'
    readonly_fields = ('file', 'status', 'report')
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'file', 'status', 'report',
            ),
        }),
    )

    def file_download(self, instance):
        if instance.file:
            return u"<a href='{}'>{}</a>".format(instance.file.url, _("Download"))
        return u""
    file_download.short_description = _("File")
    file_download.allow_tags = True

    def list_groups(self, instance):
        return u", ".join([group.name for group in instance.groups.all()])
    list_groups.short_description = _("Groups")

    def get_urls(self):
        urls = super(ImportAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^wizard/file-selection/$',
                self.admin_site.admin_view(WizardFileSelectionView.as_view()),
                name='mailup_wizard_file_selection'
            ),
            url(
                r'^wizard/groups-selection/(?P<pk>\d+)/$',
                self.admin_site.admin_view(WizardGroupsSelectionView.as_view()),
                name='mailup_wizard_groups_selection'
            ),
            url(
                r'^wizard/fields-configuration/(?P<pk>\d+)/$',
                self.admin_site.admin_view(WizardFieldsConfigurationView.as_view()),
                name='mailup_wizard_fields_configuration'
            ),
        )
        return my_urls + urls


admin.site.register(Import, ImportAdmin)
