# -*- coding: utf-8 -*-
"""Command for contacts sync in mailup console"""

import datetime
import logging
import time
from optparse import make_option

from django.core.management.base import BaseCommand
from django.conf import settings

from acomea.mailup.utils.client import MailUpClient
from acomea.contact.models import Contact, Group

log = logging.getLogger("mailup")

class Command(BaseCommand):
    """..."""
    help = 'Test mailup'
    option_list = BaseCommand.option_list + (
        make_option('-t', '--today', action="store_true", dest='today', help='Created today', default=False),
    )

    def handle(self, *scripts, **opts):
        list_id = settings.ACOMEA_MAILUP_LIST_ID
        today = opts.get('today')

        if list_id:
            print "### INIZIO SINCRONIZZAZIONE CONTATTI IN LISTA %s ###" % list_id
            # Inizializzo client che gestisce le chiamate ai servizi rest di mailup
            mailup_client = MailUpClient()
            mailup_client.retrieve_access_token()


            # Importo i contatti raggruppati per singolo gruppo (bulk insert)
            for group in Group.objects.all():
                print "Gruppo: %s" % group.name
                group_contacts = []

                qs = group.contacts.all()
                if today:
                    qs = qs.filter(date_create__gte=datetime.date.today())

                for contact in qs:
                    if contact.can_create_message[0]:
                        group_contacts.append(contact)



                if not group.mailup_group_id:
                    create_group_json = mailup_client.create_group(list_id, group.name[:50])
                    while create_group_json is None:
                        time.sleep(1)

                    group.mailup_group_id = create_group_json["idGroup"]
                    group.save()
                mailup_client.import_recipients_in_group(group.mailup_group_id, group_contacts)

            # Importo i contatti che non appartengono a nessun gruppo
            for contact in Contact.objects.filter(groups__isnull=True):
                if contact.can_create_message[0]:
                    mailup_client.import_recipient(list_id, contact)

        else:
            print "Specificare l'id della lista mailup nella quale effettuare la sincronizzazione dei contatti"