# -*- coding: utf-8 -*-
"""Command for contacts sync in mailup console"""

import os
import codecs
import datetime
import logging
import time
from HTMLParser import HTMLParser
from optparse import make_option
from shutil import copyfile
import pysftp

from django.conf import settings
from django.core.exceptions import MultipleObjectsReturned
from django.core.mail import send_mail
from django.core.management.base import BaseCommand


from acomea.contact.models import Contact, Group, Email
from acomea.mailup.utils.client import MailUpClient
from acomea.sso import exceptions

log = logging.getLogger("mailup")


class Command(BaseCommand):
    """..."""
    help = 'Test mailup'
    option_list = BaseCommand.option_list + (
        make_option(
            '-b', '--backup', action="store_true", dest='backup', help='Save csv for backup', default=False
        ),
        make_option(
            '--dry-run', action='store_true', dest='dry_run', default=False,
            help="Don't sync mailup, just the local db",
        )
    )
    mailup_client = MailUpClient()
    parser = HTMLParser()
    list_id = settings.ACOMEA_MAILUP_LIST_ID
    mailup_group_recipients = {}
    mailup_groups = dict()
    processed_emails = set()
    processed_groups = set()
    start_time = None
    dry_run = False

    # STATISTICHE PER REPORT FINALE
    righe_nel_file = 0
    contatti_acomea_presenti = 0
    contatti_acomea_creati = 0
    contatti_acomea_mailup_id_aggiornato = 0
    contatti_disiscritti_sul_sito = 0
    contatti_inseriti_in_gruppi_mailup = 0
    contatti_inseriti_in_gruppi_acomea = 0
    contatti_creati_o_modificati_su_mailup = 0
    contatti_aggiornati_ma_non_presenti_su_SSO = list()

    def handle(self, *scripts, **opts):
        backup = opts.get('backup')
        self.dry_run = opts.get('dry_run')
        log.debug(u"### INIZIO SINCRONIZZAZIONE CONTATTI IN LISTA %s ###" % self.list_id)
        log.debug(u'{}'.format(self.dry_run))
        self.start_time = datetime.datetime.now()

        self.mailup_client.retrieve_access_token()
        self.mailup_groups = self.get_mailup_groups()
        iscritti_file_name = self.get_csv_file_from_ftp('iscritti', backup)
        self.read_file(iscritti_file_name, 'process_iscritto')
        self.righe_nel_file = sum(1 for line in open(iscritti_file_name)) - 1
        os.remove(iscritti_file_name)

        self.get_subscribers_from_db()

        elapsed_time = self.start_time - datetime.datetime.now()
        emin, esec = divmod(elapsed_time.total_seconds(), 60)
        log.debug(u"Elapsed time from start: {}:{}".format(int(emin), int(esec)))

        # end-execution result
        import_result = """\n
        \tTOTAL RESULT:
                    - righe_nel_file:..............................{righe_nel_file}
                    - contatti_acomea_presenti:....................{contatti_acomea_presenti}
                    - contatti_acomea_creati:......................{contatti_acomea_creati}
                    - contatti_acomea_mailup_id_aggiornato:........{contatti_acomea_mailup_id_aggiornato}
                    - contatti_disiscritti_sul_sito:...............{contatti_disiscritti_sul_sito}
                    - contatti_inseriti_in_gruppi_mailup:..........{contatti_inseriti_in_gruppi_mailup}
                    - contatti_inseriti_in_gruppi_acomea:..........{contatti_inseriti_in_gruppi_acomea}
                    - contatti_creati_o_modificati_su_mailup:......{contatti_creati_o_modificati_su_mailup}
                    - contatti_aggiornati_ma_non_presenti_su_SSO:..{contatti_aggiornati_ma_non_presenti_su_SSO}
                      - {elenco_non_presenti_su_SSO}

                execution time: {execution_time}
LEGENDA:
- righe_nel_file: righe nel file preso via FTP su mailup
- contatti_acomea_presenti: contatti acomea la cui email (presa dal file) è presente nel DB acomea
- contatti_acomea_creati: contatti acomea la cui email (presa dal file) non è stata trovata nel DB acomea e vengono quindi creati (come "da_bonificare"). Viene anche creata anche la relativa email collegata.
- contatti_acomea_mailup_id_aggiornato: contatti acomea (fra i contatti_acomea_presenti) che sul DB acomea non hanno valorizzato il campo mailup_id. Per questi contatti viene recuperato il mailup_id da mailup e viene scritto nel DB acomea
- contatti_disiscritti_sul_sito: contatti che si sono disciscritti sul sito, la disiscrizione viene riflessa su MailUp
- contatti_inseriti_in_gruppi_mailup: contatti per cui è stato necessario un allineamento relativo alla partecipazione ai gruppi acomea->MailUp
- contatti_inseriti_in_gruppi_acomea: contatti per cui è stato necessatio un allineamento relativo alla partecipazione ai gruppi MailUp->acomea
- contatti_creati_o_modificati_su_mailup: contatti acomea da bonificare la cui email non è presente nel file e per cui viene invocato un allineamento su mailup che crea o modifica opportunamente il contatto e i campi auth_trattamento_dati e auth_materiale_commerciale
- contatti_aggiornati_ma_non_presenti_su_SSO: contatti con username presenti in db acomea, ma non presenti su sso 
- tempo di esecuzione: tempo di esecuzione
                """.format(
            righe_nel_file=self.righe_nel_file,
            contatti_acomea_presenti=self.contatti_acomea_presenti,
            contatti_acomea_creati=self.contatti_acomea_creati,
            contatti_acomea_mailup_id_aggiornato=self.contatti_acomea_mailup_id_aggiornato,
            contatti_disiscritti_sul_sito=self.contatti_disiscritti_sul_sito,
            contatti_inseriti_in_gruppi_mailup=self.contatti_inseriti_in_gruppi_mailup,
            contatti_inseriti_in_gruppi_acomea=self.contatti_inseriti_in_gruppi_acomea,
            contatti_creati_o_modificati_su_mailup=self.contatti_creati_o_modificati_su_mailup,
            contatti_aggiornati_ma_non_presenti_su_SSO=len(self.contatti_aggiornati_ma_non_presenti_su_SSO),
            elenco_non_presenti_su_SSO=u",".join(self.contatti_aggiornati_ma_non_presenti_su_SSO),
            execution_time=str(datetime.datetime.now() - self.start_time)
        )

        send_mail(
            'Acomea MailUp Synchronization report {}'.format(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")),
            '{}'.format(import_result),
            'mailupsync@acomea.it',
            settings.MAILUP_FIELDS_SYNC_RECIPIENT_REPORT_MAIL_LIST,
            fail_silently=False,
        )
        return import_result

    def get_mailup_groups(self):
        # Carica json con lista gruppi da MAILUP
        self.mailup_client.retrieve_access_token()
        groups_json = self.mailup_client.read_groups(self.list_id)
        mailup_groups, mailup_groups_ids = {}, []
        for item in groups_json["Items"]:
            mailup_groups[item['idGroup']] = item
            mailup_groups_ids.append(item['idGroup'])
        # Svuota il campo mailup_group_id per i gruppi non contenuti nella lista mailup_groups
        Group.objects.exclude(
            mailup_group_id__in=mailup_groups_ids
        ).filter(mailup_group_id__isnull=False).update(mailup_group_id=None)
        return mailup_groups

    @staticmethod
    def get_csv_file_from_ftp(name, backup=False):
        file_name = '{}{}.csv'.format(name, datetime.date.today().strftime("%Y%m%d"))
        log.debug(u"Provo a scaricare il file {}".format(file_name))
        with pysftp.Connection('ftpsrv.mailup.it', username='m47393_ftp', password='NJ9BWDVX') as sftp:
            sftp.get('ftp_export/{}'.format(file_name))
            log.debug(u"File {} scaricato".format(file_name))
        if backup:
            copyfile(file_name, '/home/www/www.acomea.it/backups/csv/{}'.format(file_name))
            log.debug(u"File {} backuppato".format(file_name))
        return file_name

    def read_file(self, file_name, command):
        with codecs.open(file_name, encoding='utf-8') as f:
            for (i, row) in enumerate(f):
                if i:
                    log.debug(u"Processo riga n. {}".format(i))
                    email = getattr(self, command)(row)
                    log.debug(u"Riga n. {} - {} processata".format(i, email))

    def process_iscritto(self, row):
        try:
            email, list_id, subscription_date, first_name, last_name, group_id_list = \
                self.parser.unescape(row.rstrip('\r\n')).split(";")
        except Exception as e:
            log.critical(u'CRITICAL: {}'.format(e))
            return
        try:
            group_id_list = [int(x) for x in group_id_list.split(',')]
        except Exception as e:
            group_id_list = []
        # Recipient
        log.debug(u"Processo contatto mailup: {} {}".format(first_name, last_name))
        # Provo a prendere il contatto dal db

        db_contact = self.get_or_create_db_contact(email, first_name, last_name)

        # se c'è il contatto
        if db_contact:

            # RACCOLTA STATISTICHE PER REPORT
            self.contatti_acomea_presenti += 1

            # se il contatto non ha il recipient id, lo prendo da mailup tramite API, perchè il csv non contiene il dato
            if not db_contact.mailup_recipient_id:

                # RACCOLTA STATISTICHE PER REPORT
                self.contatti_acomea_mailup_id_aggiornato += 1

                self.update_mailup_recipient_id(db_contact)

            # verifica se il contatto si è disiscritto sul sito e nel caso lo disiscrive anche da mailup
            is_subscribed, msg = db_contact.can_create_message
            if not is_subscribed:

                # RACCOLTA STATISTICHE PER REPORT
                self.contatti_disiscritti_sul_sito += 1

                if not self.dry_run:
                    self.mailup_client.unsubscribe_recipient(list_id, db_contact.mailup_recipient_id)
                log.debug(u"Contatto disiscritto")
                self.processed_emails.add(email)
                return email

            db_groups_set = set(db_contact.groups.values_list('mailup_group_id', flat=True))
            mailup_groups_set = set(group_id_list)

            # set contenente gli id dei gruppi cui il contatto non è iscritto su mailup
            not_on_mailup_groups_set = db_groups_set - mailup_groups_set

            if not_on_mailup_groups_set:
                # RACCOLTA STATISTICHE PER REPORT
                self.contatti_inseriti_in_gruppi_mailup += 1

            self.sync_groups_from_db(db_contact, not_on_mailup_groups_set)

            # set contenente gli id dei gruppi cui il contatto non è iscritto su db
            not_on_db_groups_set = mailup_groups_set - db_groups_set

            if not_on_mailup_groups_set:
                # RACCOLTA STATISTICHE PER REPORT
                self.contatti_inseriti_in_gruppi_acomea += 1

            self.sync_groups_from_mailup(db_contact, not_on_db_groups_set)

        else:
            # RACCOLTA STATISTICHE PER REPORT
            self.contatti_acomea_creati += 1

        log.debug(u"Contatto {} {} processato".format(first_name, last_name))
        self.processed_emails.add(email)
        return email

    def get_or_create_db_contact(self, email, first_name, last_name):
        try:
            db_contact = Contact.objects.get(emails__email__exact=email.lower())
            log.debug(u"Contatto {} {} già presente su db".format(first_name, last_name))
        # Se non esiste lo creo
        except Contact.DoesNotExist:
            db_contact = Contact.objects.create(
                first_name=first_name,
                last_name=last_name,
                bonifica=True,
                auth_trattamento_dati=True,
                auth_materiale_commerciale=True,
            )
            Email.objects.create(contact=db_contact, email=email, default=True)
            log.debug(u"Contatto creato: {} {}".format(first_name, last_name))
        # Se l'email è duplicata non sincronizzo
        except MultipleObjectsReturned:
            # TODO: gestire email duplicata
            log.error(u"ERRORE: Contatto multiplo: {} ".format(email))
            return
        return db_contact

    @staticmethod
    def convert_datetime(value, fmt='%Y%m%dT%H%M%S'):
        return datetime.datetime.strptime(value, fmt)

    def update_group(self, group):
        if group.id not in self.processed_groups:
            if not group.mailup_group_id:
                create_group_json = self.mailup_client.create_group(self.list_id, group.name)
                if create_group_json:
                    group.mailup_group_id = create_group_json["idGroup"]
                    group.save()
            else:
                if not self.dry_run:
                    self.mailup_client.update_group(self.list_id, group.mailup_group_id, group.name)
            self.processed_groups.add(group.id)

    def update_mailup_recipient_id(self, contact):
        contact.mailup_recipient_id = self.mailup_client.import_recipient(
            self.list_id, contact
        )
        try:
            contact.save()
        except exceptions.UserNotFoundOnSSOException:
            msg = u'{}'.format(contact.user.username)
            self.contatti_aggiornati_ma_non_presenti_su_SSO.append(msg)

    def get_subscribers_from_db(self):
        qs = Contact.objects.exclude(emails__email__in=self.processed_emails).filter(
            bonifica=True, auth_trattamento_dati=True, auth_materiale_commerciale=True
        )
        for db_contact in qs:
            if db_contact.can_create_message:
                if not self.dry_run:
                    # RACCOLTA STATISTICHE PER REPORT
                    self.contatti_creati_o_modificati_su_mailup += 1

                    mailup_recipient_id = self.mailup_client.import_recipient(self.list_id, db_contact)
                    old_mailup_recipient_id = None
                    if db_contact.mailup_recipient_id != mailup_recipient_id:
                        old_mailup_recipient_id = db_contact.mailup_recipient_id
                        db_contact.mailup_recipient_id = mailup_recipient_id

                    db_groups_set = set(db_contact.groups.values_list('mailup_group_id', flat=True))
                    self.sync_groups_from_db(db_contact, db_groups_set)

                    if old_mailup_recipient_id:
                        try:
                            db_contact.save()
                        except exceptions.UserNotFoundOnSSOException:
                            msg = u'{}'.format(db_contact.user.username)
                        self.contatti_aggiornati_ma_non_presenti_su_SSO.append(msg)

                log.debug(u"Iscritto contatto su mailup: {} {}".format(db_contact.first_name, db_contact.last_name))

    def sync_groups_from_db(self, contact, groups):
        for group in contact.groups.all():
            self.update_group(group)
            if group.mailup_group_id in groups and not self.dry_run:
                self.mailup_client.assign_recipient_to_group(
                    group.mailup_group_id, contact.mailup_recipient_id
                )
            time.sleep(0.2)

    def sync_groups_from_mailup(self, contact, groups):
        for mailup_group_id in groups:
            try:
                group = Group.objects.get(mailup_group_id=mailup_group_id)
            except Group.DoesNotExist:
                mailup_group = self.mailup_groups.get(mailup_group_id, None)
                if mailup_group and mailup_group.get('Name', False):
                    try:
                        group = Group.objects.get(name=mailup_group.get('Name'))

                        if group.mailup_group_id != mailup_group_id:
                            group.mailup_group_id = mailup_group_id
                            group.save()

                    except Group.DoesNotExist:
                        group = Group.objects.create(
                            mailup_group_id=mailup_group_id, name=mailup_group.get('Name')
                        )
            if group:
                contact.groups.add(group)  # signal "contact_group_signal" change "first_group" field, to use contact instace remember to reload
            else:
                log.error(u'ERRORE: Il gruppo {} deve essere verificato'.format(mailup_group_id))
