# -*- coding: utf-8 -*-

import datetime
import time
import logging
from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand

from acomea.mailup.utils.client import MailUpClient
from acomea.contact.models import Contact, Group

log = logging.getLogger("mailup")


class Command(BaseCommand):
    """
    Comando utilizzato per sincronizzare i contatti appartenenti al gruppo newsletter_risparmiamocelo
    """
    help = 'Newsletter risparmiamocelo mailup sync'
    option_list = BaseCommand.option_list + (
        make_option('-t', '--today', action="store_true", dest='today', help='Created today', default=False),
    )
    list_id = settings.ACOMEA_MAILUP_NEWSLETTER_RISPARMIAMOCELO_LIST_ID
    mailup_client = MailUpClient()

    def handle(self, *scripts, **opts):
        if self.list_id and self.list_id != settings.ACOMEA_MAILUP_LIST_ID:
            log.debug("### INIZIO SINCRONIZZAZIONE NEWSLETTER RISPARMIAMOCELO CONTATTI IN LISTA %s ###" % self.list_id)
            self.mailup_client.retrieve_access_token()
            newsletter_risparmiamocelo_group = Group.objects.get(id=settings.NEWSLETTER_RISPARMIAMOCELO_GROUP_ID)
            qs = newsletter_risparmiamocelo_group.contacts.all()
            total = qs.count()
            contact_cont = 0
            log.debug("Contatti da aggiornare: %s" % total)
            self.unsubscribe_missing_recipients(qs)
            for contact in qs:
                try:
                    contact_email = contact.get_default_email()
                    is_subscribed = contact.can_create_message_risparmiamocelo[0]
                    if contact_email and contact_email.strip():
                        log.debug("###################################################################")
                        contact_cont += 1
                        msg = "Inizio sincronizzazione contatto: {} ({}/{})".format(contact_email, contact_cont, total)
                        log.debug(msg)
                        mailup_recipient_id = self.mailup_client.import_recipient(self.list_id, contact)
                        if is_subscribed:
                            self.mailup_client.import_recipients(self.list_id, [contact], "asOptin")
                        else:
                            self.mailup_client.unsubscribe_recipient(self.list_id, mailup_recipient_id)
                except Exception, e:
                    log.debug("Eccezzione generica: %s" % e)
        else:
            log.debug("Specificare l'id della lista mailup nella quale effettuare la sincronizzazione dei contatti")

    def unsubscribe_missing_recipients(self, qs):
        recipients = self.mailup_client.read_list_recipients(self.list_id, "Subscribed")['Items']
        for recipient in recipients:
            mailup_email = recipient["Email"].strip()
            mailup_id = recipient["idRecipient"]
            try:
                contact = Contact.objects.get(emails__email__exact=mailup_email.lower())
                if contact not in qs:
                    self.mailup_client.unsubscribe_recipient(self.list_id, mailup_id)
                    log.debug("Contatto %s disiscritto da mailup" % mailup_email)
            except Exception, e:
                log.debug("Eliminazione non riuscita, errore {}".format(e))
