# -*- coding: utf-8 -*-
"""Command for contacts sync in mailup console"""

import datetime
import logging
from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand

from acomea.mailup.utils.client import MailUpClient
from acomea.contact.models import Contact, Group

log = logging.getLogger("mailup")


class Command(BaseCommand):
    """..."""
    help = 'Test mailup'
    option_list = BaseCommand.option_list + (
        make_option('-t', '--today', action="store_true", dest='today', help='Created today', default=False),
        make_option('-g', '--group', dest='group', help='Created today', default=None),
        make_option('-b', '--book', action="store_true", dest='book', help='Created today', default=False),
    )

    def handle(self, *scripts, **opts):
        list_id = settings.ACOMEA_MAILUP_LIST_ID
        today = opts.get('today')
        group_id = opts.get('group')
        book_city = opts.get('book')

        if list_id:
            log.debug("### INIZIO SINCRONIZZAZIONE CONTATTI IN LISTA %s ###" % list_id)
            # Inizializzo client che gestisce le chiamate ai servizi rest di mailup
            mailup_client = MailUpClient()
            mailup_client.retrieve_access_token()

            # Ciclo i contatti di ACOMEA
            contact_cont = 0
            qs = Contact.objects.all()
            if today:
                date_filter = datetime.date.today()-datetime.timedelta(days=2)
                qs = qs.filter(date_last_modify_for_mailup__gte=date_filter)
                log.debug("Data filtro ultima modifica contatti: %s" % date_filter)
            if group_id:
                group = Group.objects.get(id=group_id)
                qs = qs.filter(groups=group)
            if book_city:
                groups = [377, 396, 415, 416]
                qs = qs.filter(groups__id__in=groups)

            total = qs.count()
            log.debug("Contatti da aggiornare: %s" % total)
            for contact in qs:
                try:
                    contact_email = contact.get_default_email()
                    if contact_email and contact_email.strip():
                        log.debug("###################################################################")
                        contact_cont += 1
                        log.debug("Inizio sincronizzazione contatto: %s (%s/%s)" % (contact_email, contact_cont, total))
                        for status in ["Subscribed", "Unsubscribed"]:
                            check_subscriber_exists_json = mailup_client.check_subscriber_exists(
                                list_id, status, contact_email,
                            )
                            contact_exists = True if check_subscriber_exists_json["TotalElementsCount"] else False
                            if contact_exists:
                                recipient = check_subscriber_exists_json["Items"][0]
                                mailup_recipient_id = recipient["idRecipient"]
                                if mailup_recipient_id and mailup_recipient_id != contact.mailup_recipient_id:
                                    contact.contact.mailup_recipient_id = recipient["idRecipient"]
                                    contact.save()
                                break
                except Exception as e:
                    log.debug("Eccezione generica: %s" % e)
        else:
            log.debug("Specificare l'id della lista mailup nella quale effettuare la sincronizzazione dei contatti")
