# coding: utf-8
"""
Placeholder
"""

import os
import xlrd
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel
from acomea.contact.models import Group
from acomea.mailup.validators import validate_excel_extensions


class Import(UserModel, DateModel):
    file = models.FileField(
        verbose_name=_("File"),
        upload_to='mailup_imports',
        validators=[validate_excel_extensions],
    )
    status = models.CharField(
        verbose_name=_("Status"),
        choices=settings.ACOMEA_MAILUP_IMPORT_STATUS_CHOICES,
        max_length=100,
        default="pending",
    )
    mailup_status = models.CharField(
        verbose_name=_("Recipients status"),
        choices=settings.ACOMEA_MAILUP_IMPORT_RECIPIENT_STATUS_CHOICES,
        max_length=100,
        default="subscribed",
    )
    change_db_status = models.BooleanField(
        verbose_name=_("Change db status"),
        default=False,
    )
    skip_first_row = models.BooleanField(
        verbose_name=_("Skip first row"),
        default=True,
    )
    report = models.TextField(
        verbose_name=_("Report"),
        blank=True, null=True,
        default='',
    )
    fields_configuration = models.CharField(
        verbose_name=_("Fields configuration"),
        max_length=255,
        blank=True, null=True,
    )
    groups = models.ManyToManyField(
        Group,
        related_name="mailup_imports",
        verbose_name=_("Groups"),
    )
    mailup_list = models.IntegerField(
        verbose_name=_('Mailup list'),
        blank=True, null=True,
    )

    class Meta:
        ordering = ('-date_create',)
        verbose_name = _('Import')
        verbose_name_plural = _('Imports')

    def __unicode__(self):
        return u"{}".format(self.date_create.strftime("%d/%m/%Y"))

    @property
    def file_columns(self):
        if self.file:
            file_path = os.path.join(settings.MEDIA_ROOT, self.file.name)
            book = xlrd.open_workbook(file_path)
            sheet = book.sheet_by_index(0)
            return sheet.ncols
        return None

