#-*- coding: utf-8 -*-

import os
import logging
import xlrd
import simplejson
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from jmb.async.models import Task
from jmb.async.tasks import JmbJobtasticTask
from acomea.mailup.utils.helpers import create_and_import_contact

logger = logging.getLogger('celery.task')


class ImportAndSyncContactsTask(JmbJobtasticTask):
    task_name = _("Contacts import and mailup sync")
    significant_kwargs = []
    herd_avoidance_timeout = 0
    cache_duration = -1

    def calculate_result(self, instance):
        logger.info('Contacts import and mailup sync')
        file_path = os.path.join(settings.MEDIA_ROOT, instance.file.name)
        book = xlrd.open_workbook(file_path)
        sheet = book.sheet_by_index(0)
        rows_count = sheet.nrows
        cont = 0
        groups = instance.groups.all()
        # Controllo disposizione campi
        fields = simplejson.loads(instance.fields_configuration)
        for field in fields:
            email_pos = fields.index('email')
            first_name_pos = fields.index('first_name') if 'first_name' in fields else None
            last_name_pos = fields.index('last_name') if 'last_name' in fields else None
        # Ciclo righe del file
        start_row = 0 if not instance.skip_first_row else 1
        for i in range(start_row, rows_count):
            email = u"{}".format(sheet.cell(i, email_pos).value).strip()
            first_name = u"{}".format(sheet.cell(i, first_name_pos).value).strip() if first_name_pos is not None else ""
            last_name = u"{}".format(sheet.cell(i, last_name_pos).value).strip() if last_name_pos is not None else ""
            create_and_import_contact(
                instance.mailup_list, email, first_name, last_name, groups,
                mailup_status=instance.mailup_status.title(),
                change_db_status=instance.change_db_status,
            )
            self.update_progress(cont, rows_count, update_frequency=1)
            cont += 1
        # Aggiornamento stato e report importazione
        instance.status = 'completed'
        instance.report = 'Importazione completata per {} contatti'.format(cont)
        instance.save()
        # Impostazione risultati task
        task_id = self.request.id
        task = Task.objects.get(task_id=task_id)
        msg = 'Importazione completata per {} contatti'.format(cont)
        task.message = msg
        task.save()
        return {'message': msg}
