# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.validators import validate_email

from acomea.contact.models import Contact, Email
from acomea.mailup.utils.client import MailUpClient


def create_and_import_contact(
        mailup_list, email, first_name, last_name, groups, mailup_status='Subscribed', change_db_status=False
):
    try:
        validate_email(email)
    except Exception, e:
        return
    mailup_client = MailUpClient()
    mailup_client.retrieve_access_token()
    # Importazione su database
    auth_materiale_commerciale = True if mailup_status == 'Subscribed' else False
    contact_exists = Contact.objects.filter(
        emails__email__exact=email.lower()).exists()
    if not contact_exists:
        contact = Contact.objects.create(
            first_name=first_name,
            last_name=last_name,
            bonifica=True,
            auth_trattamento_dati=True,
            auth_materiale_commerciale=auth_materiale_commerciale,
        )
        Email.objects.create(contact=contact, email=email, default=True)
    else:
        contact = Contact.objects.filter(emails__email__exact=email.lower())[0]
        if change_db_status:
            contact.auth_materiale_commerciale = auth_materiale_commerciale
            if contact.auth_materiale_commerciale and not contact.auth_trattamento_dati:
                contact.auth_trattamento_dati = True
            contact.save()
    for group in groups:
        contact.groups.add(group)
        contact = contact.get_reload_from_db()  # to consider signal "contact_group_signal" manipulation
    # Importazione in mailup
    mailup_recipient_id = contact.mailup_recipient_id or mailup_client.import_recipient(mailup_list, contact)
    if contact.mailup_recipient_id != mailup_recipient_id:
        contact.mailup_recipient_id = mailup_recipient_id
        contact.save()
    if auth_materiale_commerciale:
        mailup_client.import_recipients(mailup_list, [contact], "asOptin")
    else:
        mailup_client.unsubscribe_recipient(mailup_list, mailup_recipient_id)
    if mailup_list == settings.ACOMEA_MAILUP_LIST_ID:
        for group in groups:
            if not group.mailup_group_id:
                create_group_json = mailup_client.create_group(mailup_list, group.name)
                group.mailup_group_id = create_group_json["idGroup"]
                group.save()
            else:
                mailup_client.update_group(mailup_list, group.mailup_group_id, group.name)
            mailup_client.assign_recipient_to_group(group.mailup_group_id, mailup_recipient_id)
