# -*- coding: utf-8 -*-

import os
import xlrd

from django.conf import settings
from django.core.urlresolvers import reverse
from django.views.generic import CreateView, UpdateView

from acomea.mailup.forms import WizardFileSelectionForm, WizardGroupsSelectionForm, WizardFieldsConfigurationForm
from acomea.mailup.models import Import


class WizardFileSelectionView(CreateView):
    template_name = 'admin/mailup/import/wizard_file_selection.html'
    form_class = WizardFileSelectionForm

    def get_success_url(self):
        return reverse('admin:mailup_wizard_groups_selection', args=(self.object.id,))


class WizardGroupsSelectionView(UpdateView):
    template_name = 'admin/mailup/import/wizard_groups_selection.html'
    form_class = WizardGroupsSelectionForm
    model = Import

    def get_success_url(self):
        return reverse('admin:mailup_wizard_fields_configuration', kwargs={'pk': self.object.id})


class WizardFieldsConfigurationView(UpdateView):
    template_name = 'admin/mailup/import/wizard_fields_configuration.html'
    form_class = WizardFieldsConfigurationForm
    model = Import

    def get_context_data(self, **kwargs):
        context = super(WizardFieldsConfigurationView, self).get_context_data(**kwargs)
        file_path = os.path.join(settings.MEDIA_ROOT, self.object.file.name)
        book = xlrd.open_workbook(file_path)
        sheet = book.sheet_by_index(0)
        rows_count, columns_count = sheet.nrows, sheet.ncols
        table_rows = []
        for row_pos in range(0, rows_count):
            row = []
            for col_pos in range(0, columns_count):
                row.append(u"{}".format(sheet.cell(row_pos, col_pos).value))
            table_rows.append(row)
        context.update({
            'table_rows': table_rows,
        })
        return context

    def get_form_kwargs(self):
        kwargs = super(WizardFieldsConfigurationView, self).get_form_kwargs()
        kwargs['request'] = self.request
        return kwargs

    def get_success_url(self):
        return reverse('admin:mailup_import_changelist')

